# Detect the distribution in use
%global __despace head -n 1 | tr -d '[:space:]' | sed -e 's/[(].*[)]//g'
%global __lower4 cut -c 1-4 | tr '[:upper:]' '[:lower:]'
%global __distfile %([ -f /etc/SuSE-release ] && echo /etc/SuSE-release || echo /etc/redhat-release)
%global __distinit %(sed -e 's/ release .*//' -e 's/\\([A-Za-z]\\)[^ ]*/\\1/g' %{__distfile} | %{__despace} | %{__lower4})
%global __distvers %(sed -e 's/.* release \\([^. ]*\\).*/\\1/' %{__distfile} | %{__despace})
# Identify Alma, CentOS, CentOS Stream and Rocky Linux as rhel
%if "%{__distinit}" == "a" || "%{__distinit}" == "c" || "%{__distinit}" == "cl" || "%{__distinit}" == "cs" || "%{__distinit}" == "rl"
%global __distinit rhel
%endif
# Dist tag for Fedora is still "fc"
%if "%{__distinit}" == "f"
%global __distinit fc
%endif

# Build --with debug_valgrind for multi-arch build and additional valgrind debugging
%bcond_with debug_valgrind

# A noarch-turned-arch package should not have debuginfo
%global debug_package %{nil}

Name:		perl-Test-Valgrind
Summary:	Generate suppressions, analyze and test any command with valgrind
Version:	1.19
Release:	23.%{__distinit}%{__distvers}
License:	GPL-1.0-or-later OR Artistic-1.0-Perl
URL:		https://metacpan.org/release/Test-Valgrind
Source0:	https://cpan.metacpan.org/modules/by-module/Test/Test-Valgrind-%{version}.tar.gz
Patch1:		Test-Valgrind-1.19-Perl_pp_entersub.patch
%if !%{with debug_valgrind}
BuildArch:	noarch
%endif
# Module Build
BuildRequires:	coreutils
BuildRequires:	findutils
BuildRequires:	gcc
BuildRequires:	make
BuildRequires:	perl-devel
BuildRequires:	perl-generators
BuildRequires:	perl-interpreter
BuildRequires:	perl(ExtUtils::Install) >= 1.38
BuildRequires:	perl(ExtUtils::MakeMaker)
BuildRequires:	sed
# Module Runtime
BuildRequires:	perl(base)
BuildRequires:	perl(Carp)
BuildRequires:	perl(Config)
BuildRequires:	perl(Digest::MD5)
BuildRequires:	perl(DynaLoader)
BuildRequires:	perl(Env::Sanctify)
BuildRequires:	perl(ExtUtils::MM)
BuildRequires:	perl(Fcntl)
BuildRequires:	perl(File::HomeDir) >= 0.86
BuildRequires:	perl(File::Path)
BuildRequires:	perl(File::Spec)
BuildRequires:	perl(File::Temp) >= 0.19
BuildRequires:	perl(Filter::Util::Call)
BuildRequires:	perl(IO::Select)
BuildRequires:	perl(List::Util)
BuildRequires:	perl(overload)
BuildRequires:	perl(Perl::Destruct::Level)
BuildRequires:	perl(POSIX)
BuildRequires:	perl(Scalar::Util)
BuildRequires:	perl(strict)
BuildRequires:	perl(Test::Builder)
BuildRequires:	perl(warnings)
BuildRequires:	perl(XML::Twig)
BuildRequires:	perl(XML::Twig::Elt)
BuildRequires:	valgrind >= 3.1.0
# Test Suite
BuildRequires:	perl(lib)
BuildRequires:	perl(Test::More)
BuildRequires:	perl(Time::HiRes)
BuildRequires:	perl(XSLoader)
# Dependencies
%if 0%{?fedora} < 38 && 0%{?rhel} < 10
Requires:	perl(:MODULE_COMPAT_%(eval "`perl -V:version`"; echo $version))
%endif
Requires:	perl(Carp)
Requires:	perl(Config)
Requires:	perl(Digest::MD5)
Requires:	perl(DynaLoader)
Requires:	perl(File::HomeDir) >= 0.86
Requires:	perl(File::Path)
Requires:	perl(File::Temp) >= 0.14
Requires:	perl(Filter::Util::Call)
Requires:	perl(Perl::Destruct::Level)
Requires:	perl(XML::Twig)
Requires:	perl(XML::Twig::Elt)
Requires:	valgrind >= 3.1.0

%description
The Test::Valgrind::* API lets you run Perl code through the memcheck tool of
the valgrind memory debugger, to test for memory errors and leaks. The
Test::Valgrind module itself is a front-end to this API. If they aren't
available yet, it will first generate suppressions for the current perl
interpreter and store them in the portable flavor of
~/.perl/Test-Valgrind/suppressions/$VERSION. The actual run will then take
place, and tests will be passed or failed according to the result of the
analysis.

The complete API is much more versatile than this. By declaring an appropriate
Test::Valgrind::Command class, you can run any executable (that is, not only
Perl scripts) under valgrind, generate the corresponding suppressions
on-the-fly and convert the analysis result to TAP output so that it can be
incorporated into your project's test suite. If you're not interested in
producing TAP, you can output the results in whatever format you like (for
example HTML pages) by defining your own Test::Valgrind::Action class.

%prep
%setup -q -n Test-Valgrind-%{version}

# Without debuginfo, the symbol 'Perl_pp_entersub' is not always
# appearing in the valgrind trace report, causing t/20-bad.t to fail
# as a result of not recognizing the trace record
#
# This is a workaround to help the test identify the trace correctly
%patch -P 1

# Avoid doc-file deps and fix shellbangs
sed -i -e 's|^#!/usr/bin/env perl|#!/usr/bin/perl|' samples/map.pl
chmod -c -x samples/map.pl

%if %{with debug_valgrind}
# Create a wrapper script for valgrind so we can see how it's being used
mkdir bin
cat << 'EOF' > bin/valgrind
#!/bin/bash

echo "### valgrind " "$@" >> valgrind.output
/usr/bin/valgrind "$@" | tee -a valgrind.output
EOF
chmod 755 bin/valgrind
%endif

%build
perl Makefile.PL INSTALLDIRS=vendor OPTIMIZE="%{optflags}"
make %{?_smp_mflags}

%install
make pure_install DESTDIR=%{buildroot}
find %{buildroot} -type f -name .packlist -delete
find %{buildroot} -type f -name '*.bs' -empty -delete
%{_fixperms} -c %{buildroot}

# The package is noarch; the XS code included is for testing purposes and is
# not part of the module itself
if [ "%{perl_vendorarch}" != "%{perl_vendorlib}" ]; then
	mkdir -p %{buildroot}%{perl_vendorlib}
	mv %{buildroot}%{perl_vendorarch}/* %{buildroot}%{perl_vendorlib}/
fi

%check
%if %{with debug_valgrind}
# Pick up our local valgrind script
PATH=$(pwd)/bin:$PATH
%endif
make test

%files
%doc Changes README samples/
%{perl_vendorlib}/Test/
%{_mandir}/man3/Test::Valgrind.3*
%{_mandir}/man3/Test::Valgrind::Action.3*
%{_mandir}/man3/Test::Valgrind::Action::Captor.3*
%{_mandir}/man3/Test::Valgrind::Action::Suppressions.3*
%{_mandir}/man3/Test::Valgrind::Action::Test.3*
%{_mandir}/man3/Test::Valgrind::Carp.3*
%{_mandir}/man3/Test::Valgrind::Command.3*
%{_mandir}/man3/Test::Valgrind::Command::Aggregate.3*
%{_mandir}/man3/Test::Valgrind::Command::Perl.3*
%{_mandir}/man3/Test::Valgrind::Command::PerlScript.3*
%{_mandir}/man3/Test::Valgrind::Component.3*
%{_mandir}/man3/Test::Valgrind::Parser.3*
%{_mandir}/man3/Test::Valgrind::Parser::Suppressions::Text.3*
%{_mandir}/man3/Test::Valgrind::Parser::Text.3*
%{_mandir}/man3/Test::Valgrind::Parser::XML.3*
%{_mandir}/man3/Test::Valgrind::Parser::XML::Twig.3*
%{_mandir}/man3/Test::Valgrind::Report.3*
%{_mandir}/man3/Test::Valgrind::Session.3*
%{_mandir}/man3/Test::Valgrind::Suppressions.3*
%{_mandir}/man3/Test::Valgrind::Tool.3*
%{_mandir}/man3/Test::Valgrind::Tool::memcheck.3*
%{_mandir}/man3/Test::Valgrind::Util.3*
%{_mandir}/man3/Test::Valgrind::Version.3*

%changelog
* Sun Jul 30 2023 Paul Howarth <paul@city-fan.org> - 1.19-23
- Fix FTBFS in Fedora 39 due to failing t/20-bad.t (rhbz#2222854)

* Wed May 17 2023 Paul Howarth <paul@city-fan.org> - 1.19-22
- Use SPDX-format license tag

* Sat Jun 27 2020 Paul Howarth <paul@city-fan.org> - 1.19-14
- Perl 5.32 rebuild

* Sat Jun 30 2018 Paul Howarth <paul@city-fan.org> - 1.19-7
- Perl 5.28 rebuild

* Tue Feb 13 2018 Paul Howarth <paul@city-fan.org> - 1.19-6
- Add build option --with debug_valgrind
- Fix shellbang in samples/map.pl
- BR: perl-generators unconditionally
- BR: perl-interpreter rather than perl

* Tue Jun 13 2017 Paul Howarth <paul@city-fan.org> - 1.19-3
- Drop support for EOL distributions prior to F-13
  - Drop BuildRoot: and Group: tags
  - Drop explicit buildroot cleaning in %%install section
  - Drop explicit %%clean section
  - BR: perl-devel unconditionally
  - Drop support for building with File::Temp < 0.19 and
    ExtUtils::Install < 1.3702

* Tue Aug  2 2016 Paul Howarth <paul@city-fan.org> - 1.19-1
- Update to 1.19
  - valgrind 3.1x will no longer be treated as valgrind 3.1.0, causing the
    wrong command line arguments to be used (Debian bug 832833)
  - The number of callers is now capped at 24, as this is the maximum number
    supported by valgrind; a higher number of frames could lead to the
    generation of unusable suppressions in both old and recent versions of
    valgrind
- BR: perl-generators where available
- Simplify find commands using -empty and -delete

* Mon Nov 16 2015 Paul Howarth <paul@city-fan.org> - 1.18-1
- Update to 1.18
  - A new Test::Valgrind::Version class has been added to represent valgrind
    version numbers, instead of lazily relying on version.pm
  - The detection of the valgrind executable has been slightly improved to
    cover some edge cases
  - Test failures on Windows, or with old versions of Test::More or File::Temp,
    have been addressed
  - A few extraneous warnings displayed by some tests when they were run with
    old versions of Test::Harness were silenced

* Fri Nov 13 2015 Paul Howarth <paul@city-fan.org> - 1.17-1
- Update to 1.17
  - The Test::Valgrind tests will now be skipped when the default and
    user-supplied suppressions files do not refer to any perl-related symbol;
    this behaviour can be overridden by passing 'allow_no_supp => 1' to
    Test::Valgrind->import
  - Fix CPAN RT#101934: t/20-bad.t failing on armv7hl; while the root cause of
    this issue is probably not at Test::Valgrind's level, it should
    nevertheless not run the tests when the suppression files are obviously
    insufficient
  - The accuracy of the default perl suppression file has been improved
  - The tests will be more reliably skipped when no valgrind or no suppressions
    are found
  - Segmentation faults during the analysis are now more gracefully handled
  - 'no_def_supp => 1' will no longer cause the extra suppressions to be
    ignored
  - t/20-bad.t will no longer run the extra tests when no valgrind can be found
    (this was a regression in version 1.15)
- Bundle a recent enough copy of File::Temp for testing on old distributions
- Add test patch for compatibility with old Test::Builder versions
- Drop redundant %%{?perl_default_filter}

* Sat Oct 31 2015 Paul Howarth <paul@city-fan.org> - 1.15-1
- Update to 1.15
  - The new 'regen_def_supp' option can be passed to Test::Valgrind->import to
    forcefully regenerate the default suppression file
  - Fix build failures of the dummy XS code with PERL_IMPLICIT_SYS perls
  - Fix handshake failures in tests with recent perls built with PERL_POISON
  - Test: Improved diagnostics on failure
  - Freshen Makefile.PL
- Classify buildreqs by usage

* Thu Sep 11 2014 Paul Howarth <paul@city-fan.org> - 1.14-3
- Drop %%defattr, redundant since rpm 4.4

* Sun Sep  1 2013 Paul Howarth <paul@city-fan.org> - 1.14-1
- Update to 1.14
  - This is a maintenance release; the code contains no functional change
  - POD headings are now properly linkable
  - Author tests are no longer bundled with this distribution
  - The stack traces used in t/20-bad.t have been made more predictable when
    compiler optimizations are in use
- Drop buildreqs only needed for author tests

* Mon Jul 22 2013 Paul Howarth <paul@city-fan.org> - 1.13-8
- Perl 5.18 rebuild

* Mon Jul 16 2012 Paul Howarth <paul@city-fan.org> - 1.13-5
- Perl 5.16 post-bootstrap rebuild

* Sun Jul  8 2012 Paul Howarth <paul@city-fan.org> - 1.13-4
- Perl 5.16 rebuild

* Thu May  3 2012 Paul Howarth <paul@city-fan.org> - 1.13-3
- Incorporate suggestions from package review (#803057)
  - BR: perl(Pod::Coverage) ≥ 0.18
  - BR: perl(XSLoader)
  - BR: at least version 1.22 of perl(Test::Pod)
  - BR: at least version 1.08 of perl(Test::Pod::Coverage)

* Mon Mar 12 2012 Paul Howarth <paul@city-fan.org> - 1.13-1
- Initial RPM version
